/* CEClassEditor.h				 
 *
 * This object controls the data of a beaker (molecules, cameras, groups etc.)
 * It is the main document of BeakerBoy and controls everything from loading to
 * setting up the browser which does most of the other work.
 *
 * Usage: ....
 *
 * Written by: 		Thomas Engel
 * Created:    		23.10.1993 (Copyleft)
 * Last modified: 	12.11.1994
 */

#define CE_FILETYPE_NONE		0
#define CE_FILETYPE_ASCII		1
#define CE_FILETYPE_RTF			2
#define CE_FILETYPE_RTFD		3

#import <appkit/appkit.h>
#import "CEAppManager.h"

@interface CEClassEditor:Object
{
	id	window;
	id	cheatWindow;
	id	browser;
	
	id	classHierarchyField;
	id	popUp;
	
	id	selectorNameField;
	id	methodNameField;
	id	categoryField;
	id	docuTextView;
	id	sourceTextView;
	id	seeAlsoField;
	
	id	browserModePopup;
	id	methodTypeSwitches;
	id	accessTypeSwitches;
	id	displayTypeSwitches;	
	
	id	filename;
	id	methodList;
	
	id	headerFile;
	id	sourceFile;
	id	docuFile;
	int	headerStyle;
	int	sourceStyle;
	int	docuStyle;
	
	@private
	
	BOOL	_updatingTextViews;
	BOOL	_changedTexts;
	id		_changedWindow;
	int		_docuOriginStart;
	int		_docuOriginEnd;
	int		_sourceOriginStart;
	int		_sourceOriginEnd;

	id		_tempPb;
}

+ initialize;

- init;
- initFromFile:(const char *)fileName;
- _try:path withAlternatives:anArray forText:aText andGetStyle:(int *)aStyle;
- _readFile:path forText:aText andGetStyle:(int *)aStyle;
- (BOOL)_isFileRTF:path;
- free;

- (const char *)filename;
- window;

- reparseMethods:sender;
- _parseMethodFile;
- save:sender;
- _writeText:aText withStyle:(int)aStyle to:path;
- close:sender;
- undo:sender;

// Keeping windows in sync...

- switchToNewDisplayMode:sender;
- selectNewMethod:sender;
- _setIsUpdatingTextViews:(BOOL)flag;
- _showSourceForMethod:aMethod;
- (BOOL)_selectSourceForMethod:aMethod;
- _silentlySyncWindows;
- _syncWindows;

// Working with documentation...

- checkDocumentation:sender;
- _showDocuForMethod:aMethod;
- (BOOL)_selectDocuForMethod:aMethod;
- _addDocuForMethod:aMethod;

// Controlling the windows

- showCheatWindow:sender;
- showDocumentationOnly:sender;
- showInterfaceOnly:sender;
- showImplementationOnly:sender;

// Some delegate methods.

- (int)browser:sender fillMatrix:matrix inColumn:(int)column;
- browser:sender loadCell:cell atRow:(int)row inColumn:(int)column;

- textDidChange:sender;
- textDidGetKeys:sender isEmpty:(BOOL)flag;
- _highlightPrevious:(char *)aString inView:aText;
- textShouldPerformCompletion:sender;

- windowDidBecomeKey:sender;
- windowWillClose:sender;

@end

/*
 * History: 13.01.95 Buh
 *			
 *
 * Bugs: - ...
 */